import { SemanticMark } from "./semantic-mark";

import { GrammarMarkType, Factory } from "@visactor/vgrammar-core";

import { PlotMakType } from "./enums";

import { field as getFieldAccessor } from "@visactor/vgrammar-util";

export class CirclePackingSemanticMark extends SemanticMark {
    constructor(id) {
        super(PlotMakType.circlePacking, id), Factory.getTransform(PlotMakType.circlePacking) || this._logger.error("Please add this line of code: import { registerCirclePackingTransforms } from '@visactor/vgrammar-hierarchy'; \n        and run registerCirclePackingTransforms() before use treemap chart");
    }
    setMarkType() {
        return GrammarMarkType.circle;
    }
    setDefaultDataTransform() {
        var _a;
        return [ {
            type: PlotMakType.circlePacking,
            width: {
                signal: "viewWidth"
            },
            height: {
                signal: "viewHeight"
            },
            nodeKey: null === (_a = this.spec.encode) || void 0 === _a ? void 0 : _a.node,
            flatten: !0
        } ];
    }
    parseScaleByEncode(channel, option) {
        return "color" === channel ? {
            type: "ordinal",
            id: this.getScaleId("color"),
            domain: {
                data: this.getDataIdOfFiltered(),
                field: option
            },
            range: this.getPalette()
        } : null;
    }
    convertMarkEncode(encode) {
        var _a, _b, _c;
        const markEncoder = this.convertSimpleMarkEncode(encode), res = {
            x: {
                field: "x"
            },
            y: {
                field: "y"
            },
            radius: {
                field: "radius"
            }
        };
        if (markEncoder.stroke && (res.stroke = markEncoder.stroke), markEncoder.color) {
            const scaleColorId = this.getScaleId("color"), colorAccessor = getFieldAccessor(markEncoder.color.field);
            res.fill = (datum, el, params) => {
                const scale = params[scaleColorId];
                return (null == datum ? void 0 : datum.datum) ? scale.scale(colorAccessor(datum.datum[datum.datum.length - 1])) : void 0;
            };
        } else res.fill = null !== (_b = null === (_a = this.spec.style) || void 0 === _a ? void 0 : _a.fill) && void 0 !== _b ? _b : null === (_c = this.getPalette()) || void 0 === _c ? void 0 : _c[0];
        return res;
    }
    setMainMarkSpec() {
        return {
            key: "key"
        };
    }
    setLabelTextGetter(channel, option) {
        const textGetter = getFieldAccessor(channel);
        return (datum, el, params) => textGetter(datum.datum[datum.datum.length - 1]);
    }
}

CirclePackingSemanticMark.type = PlotMakType.circlePacking;
//# sourceMappingURL=circle-packing.js.map