import { field as getFieldAccessor } from "@visactor/vgrammar-util";

import { SemanticMark } from "./semantic-mark";

import { GrammarMarkType } from "@visactor/vgrammar-core";

import { isArray, maxInArray, minInArray } from "@visactor/vutils";

import { PlotMakType } from "./enums";

import { isContinuous } from "@visactor/vscale";

export class RectSemanticMark extends SemanticMark {
    constructor(id) {
        super(PlotMakType.rect, id);
    }
    setMarkType() {
        return GrammarMarkType.rect;
    }
    parseScaleByEncode(channel, option) {
        return this.parseScaleOfCommonEncode(channel, option);
    }
    setDefaultCrosshair() {
        return {
            x: {
                crosshairShape: "rect"
            }
        };
    }
    setDefaultTooltip() {
        var _a;
        const encodeY = null === (_a = this.spec.encode) || void 0 === _a ? void 0 : _a.y;
        return {
            content: isArray(encodeY) ? encodeY.map((entry => ({
                value: entry
            }))) : [ {
                value: encodeY
            } ]
        };
    }
    convertMarkEncode(encode) {
        var _a, _b, _c, _d, _e, _f;
        const markEncoder = this.convertSimpleMarkEncode(encode), scaleXId = this.getScaleId("x"), scaleYId = this.getScaleId("y"), res = {};
        if (isArray(null === (_a = markEncoder.x) || void 0 === _a ? void 0 : _a.field)) res.x = {
            field: markEncoder.x.field[0],
            scale: markEncoder.x.scale
        }, res.x1 = {
            field: markEncoder.x.field[1],
            scale: markEncoder.x.scale
        }; else {
            const xAccessor = getFieldAccessor(markEncoder.x.field);
            res.x = (datum, el, params) => {
                const xVals = xAccessor(datum), scale = params[scaleXId];
                return isArray(xVals) ? scale.scale(xVals[0]) : scale.scale(xVals);
            }, res.x1 = (datum, el, params) => {
                var _a, _b, _c, _d, _e;
                const scale = params[scaleXId], xVals = xAccessor(datum);
                if (isArray(xVals) && xVals.length > 1) return scale.scale(xVals[1]);
                if (isContinuous(scale.type)) {
                    const domain = null !== (_a = scale.domain()) && void 0 !== _a ? _a : [], min = minInArray(domain), max = maxInArray(domain), baseValue = min > 0 ? min : max < 0 ? max : 0;
                    return scale.scale(baseValue);
                }
                return scale.scale(xVals) + (null !== (_e = null !== (_c = null === (_b = scale.bandwidth) || void 0 === _b ? void 0 : _b.call(scale)) && void 0 !== _c ? _c : null === (_d = scale.step) || void 0 === _d ? void 0 : _d.call(scale)) && void 0 !== _e ? _e : 0);
            };
        }
        if (isArray(null === (_b = markEncoder.y) || void 0 === _b ? void 0 : _b.field)) res.y = {
            field: markEncoder.y.field[0],
            scale: markEncoder.y.scale
        }, res.y1 = {
            field: markEncoder.y.field[1],
            scale: markEncoder.y.scale
        }; else {
            const yAccessor = getFieldAccessor(markEncoder.y.field);
            res.y = (datum, el, params) => {
                const yVals = yAccessor(datum), scale = params[scaleYId];
                return isArray(yVals) ? scale.scale(yVals[0]) : scale.scale(yVals);
            }, res.y1 = (datum, el, params) => {
                var _a, _b, _c, _d;
                const scale = params[scaleYId], yVals = yAccessor(datum);
                if (isArray(yVals) && yVals.length > 1) return scale.scale(yVals[1]);
                if (isContinuous(scale.type)) {
                    const domain = scale.domain(), min = minInArray(domain), max = maxInArray(domain), baseValue = min > 0 ? min : max < 0 ? max : 0;
                    return scale.scale(baseValue);
                }
                return scale.scale(yVals) + (null !== (_d = null !== (_b = null === (_a = scale.bandwidth) || void 0 === _a ? void 0 : _a.call(scale)) && void 0 !== _b ? _b : null === (_c = scale.step) || void 0 === _c ? void 0 : _c.call(scale)) && void 0 !== _d ? _d : 0);
            };
        }
        return markEncoder.stroke && (res.stroke = markEncoder.stroke), markEncoder.color || markEncoder.group ? res.fill = null !== (_c = markEncoder.color) && void 0 !== _c ? _c : markEncoder.group : res.fill = null !== (_e = null === (_d = this.spec.style) || void 0 === _d ? void 0 : _d.fill) && void 0 !== _e ? _e : null === (_f = this.getPalette()) || void 0 === _f ? void 0 : _f[0], 
        res;
    }
}

RectSemanticMark.type = PlotMakType.rect;
//# sourceMappingURL=rect.js.map